from __future__ import print_function, unicode_literals

import sys
import traceback
import github_migration
import jenkins_migration
import make_private
import update_topics_for_team
import find_new_git_url

from PyInquirer import style_from_dict, Token, prompt


def _main():
    try:
        style = style_from_dict({
            Token.QuestionMark: '#E91E63 bold',
            Token.Selected: '#673AB7 bold',
            Token.Instruction: '',  # default
            Token.Answer: '#2196f3 bold',
            Token.Question: '',
        })

        print('Github Migration')
        chose_flow = [
            {
                'type': 'list',
                'name': 'selectFlow',
                'message': 'What do you want to do?',
                'choices': [
                    'Github Migration',
                    'Jenkins BMX Migration',
                    'Make repos private',
                    'Update topics',
                    'Find new git URL for CI/CD changes (SNOW)'
                ]
            }
        ]

        chose_flow_answers = prompt(chose_flow, style=style)

        if chose_flow_answers.get("selectFlow") == "Github Migration":

            questions = [
                {
                    'type': 'input',
                    'name': 'source_github_org',
                    'message': 'Source github org',
                    'default': 'nike-platform-fulfillment'
                },
                {
                    'type': 'input',
                    'name': 'target_github_org',
                    'message': 'Target github org',
                    'default': 'nike-internal'
                },
                {
                    'type': 'input',
                    'name': 'target_team_name',
                    'message': 'Target Team Slug',
                    'default': 'fulfillment-squad-artemis'
                },
                {
                    'type': 'input',
                    'name': 'target_team_admin',
                    'message': 'Target Team Admin Slug',
                    'default': 'fulfillment-admins'
                },
                {
                    'type': 'input',
                    'name': 'prefix',
                    'message': 'Team Prefix',
                    'default': 'fulfillment'
                },
                {
                    'type': 'password',
                    'name': 'github_password',
                    'message': 'Github access token',

                }
            ]
            answers = prompt(questions, style=style)

            chose_search = [
                {
                    'type': 'list',
                    'name': 'search_criteria',
                    'message': 'How do you want to get the repos?',
                    'choices': [
                        'Search by Topic',
                        'Provide list of Repos'
                    ]
                }
            ]

            chose_search_answers = prompt(chose_search, style=style)

            if chose_search_answers.get("search_criteria") == "Search by Topic":

                questions = [
                    {
                        'type': 'input',
                        'name': 'topic_name',
                        'message': 'Topic Name. Enter multiple separated by comma',
                        'default': 'artemis'
                    },
                ]
                topic_answers = prompt(questions, style=style)
            else:
                questions = [
                    {
                        'type': 'input',
                        'name': 'filepath',
                        'message': 'Provide the file with the repo names to be migrated',
                        'default': '/Users/vmari2/repos.txt'
                    },
                ]
                topic_answers = prompt(questions, style=style)

            github_migration.migrate_github(
                answers, topic_answers
            )
        elif chose_flow_answers.get("selectFlow") == "Make repos private":
            make_private_questions = [{
                'type': 'input',
                'name': 'source_org',
                'message': 'Organization Name',
                'default': "nike-platform-logistics"
            },
                {
                    'type': 'password',
                    'name': 'github_token',
                    'message': 'github personal access token'
                }]
            make_private_answers = prompt(questions=make_private_questions, style=style)
            make_private.make_private_flow(make_private_answers)

        elif chose_flow_answers.get("selectFlow") == "Update topics":

            update_topics_questions = [{
                'type': 'input',
                'name': 'source_org',
                'message': 'Organization Name',
                'default': "nike-platform-logistics"
            },
                {
                    'type': 'input',
                    'name': 'team_slug',
                    'message': 'Team Slug',
                    'default': "logistics-squad-airmax"
                },
                {
                    'type': 'list',
                    'name': 'permission_level',
                    'message': 'Select the permission level',
                    'choices': [
                        'push',
                        'pull',
                        'admin',
                        'maintain'
                    ]
                },
                {
                    'type': 'input',
                    'name': 'additional_topics',
                    'message': 'Additional topics, comma separated',
                    'default': "logistics,airmax"
                },
                {
                    'type': 'password',
                    'name': 'github_token',
                    'message': 'github personal access token'
                }]
            update_topics_answers = prompt(update_topics_questions, style=style)
            update_topics_for_team.update_topics(update_topics_answers)
        elif chose_flow_answers.get("selectFlow") == "Find new git URL for CI/CD changes (SNOW)":
            change_url_questions = [
                {
                    'type': 'input',
                    'name': 'source_org',
                    'message': 'Source github organization',
                    'default': "nike-platform-fulfillment"
                },
                {
                    'type': 'input',
                    'name': 'path_to_data',
                    'message': 'File path for the list of Old URLs',
                    'default': "/Users/XXXX/test.txt"
                },
                {
                    'type': 'password',
                    'name': 'github_token',
                    'message': 'Github Token'
                },
            ]

            find_new_git_url_answers = prompt(change_url_questions, style=style)
            find_new_git_url.find_urls(find_new_git_url_answers)

        else:
            jenkins_questions = [{
                'type': 'confirm',
                'name': 'update_jenkins_job',
                'message': 'Update Jenkins Job with new repo',
                'default': True
            }]
            jenkins_questions_answers = prompt(jenkins_questions, style=style)

            if jenkins_questions_answers.get('update_jenkins_job'):
                questions = [

                    {
                        'type': 'input',
                        'name': 'jenkins_url',
                        'message': 'BMX Jenkins URL'
                    },
                    {
                        'type': 'password',
                        'name': 'jenkins_token',
                        'message': 'Jenkins Token'
                    },
                    {
                        'type': 'input',
                        'name': 'jenkins_username',
                        'message': 'Jenkins Username',
                        'default': 'change_it@nike.com'
                    },
                    {
                        'type': 'input',
                        'name': 'new_org',
                        'message': 'New Github Org',
                        'default': 'nike-internal'
                    },
                    {
                        'type': 'input',
                        'name': 'old_org',
                        'message': 'Old Github Org(s). Provide comma separated values ',
                        'default': 'mp-commerce-fulfillment'
                    },
                    {
                        'type': 'input',
                        'name': 'team_prefix',
                        'message': 'Team Prefix',
                        'default': 'fulfillment'
                    },
                    {
                        'type': 'input',
                        'name': 'new_credentials_id',
                        'message': 'Provide the new credentials to Change credentials id. Leave blank if not changing ',
                        'default': ''
                    }
                ]
                jenkins_answers = prompt(questions, style=style)
                jenkins_migration.get_jobs(jenkins_answers)

    except Exception as ex:
        traceback.print_exc()
        print(ex)


def main():
    return _main()


if __name__ == "__main__":
    sys.exit(_main())



import requests
from requests.auth import HTTPBasicAuth

import jenkins
import json
from PyInquirer import style_from_dict, Token, prompt
import os
import re

style = style_from_dict({
    Token.QuestionMark: '#E91E63 bold',
    Token.Selected: '#673AB7 bold',
    Token.Instruction: '',  # default
    Token.Answer: '#2196f3 bold',
    Token.Question: '',
})


def get_jobs(jenkins_answers):
    jenkins_url = jenkins_answers.get("jenkins_url")
    username = jenkins_answers.get("jenkins_username")
    token = jenkins_answers.get("jenkins_token")

    team_prefix = jenkins_answers.get("team_prefix")
    old_org = jenkins_answers.get("old_org")
    new_org = jenkins_answers.get("new_org")

    api_uri = "<apiUri>https://github.nike.com/api/v3<apiUri>"

    server = jenkins.Jenkins(jenkins_url, username=username, password=token)

    credentialsId = jenkins_answers.get("new_credentials_id")
    jobs = server.get_jobs(folder_depth=5)
    is_prompt_user = True
    for job in jobs:
        if job.get("_class") in ("org.jenkinsci.plugins.workflow.multibranch.WorkflowMultiBranchProject",
                                 "org.jenkinsci.plugins.workflow.job.WorkflowJob"):
            config = server.get_job_config(job.get("name"))
            config = config.replace(old_org + "/", f"{new_org}/{team_prefix}.").replace(api_uri,
                                                                                        "<apiUri>https://api.github.com<apiUri>")
            config = config.replace("github.nike.com", "github.com")
            for org in old_org.split(","):
                config = config.replace(org, new_org)
            if credentialsId.strip() != "":
                config = re.sub("<credentialsId>.*</credentialsId>", f"<credentialsId>{credentialsId}</credentialsId>",
                                config)
            # job_name = job.get("name")

            # change_name = [
            #
            #     {
            #         'type': 'confirm',
            #         'name': 'change_name',
            #         'message': f"Change the bmx job name {job_name} to team_prefix.<oldname>?"
            #     }
            # ]
            #
            # change_name_answers = prompt(change_name, style=style)
            #
            # if change_name_answers.get("change_name"):
            #     job_match = re.findall(r'<displayName>.*</displayName>', config)
            #     if len(job_match)>0:
            #         job_name = job_match[0].replace("displayName", "").replace("<>", "").replace("</>", "")
            #
            #         new_job_name = f"{team_prefix}.{job_name}"
            #         config = re.sub("<displayName>.*</displayName>", f"<displayName>{new_job_name}</displayName>", config)
            #     else:
            #         print("unable to change name")

            with open(job.get("name"), "w") as outfile:
                outfile.write(config)
                job_name = job.get("name")
                server.reconfig_job(job.get("name"), config_xml=config)
            os.remove(job.get("name"))
            print(f"completed configuring {job_name}")

            if is_prompt_user:
                    is_okay_to_proceed = [
                        {
                            'type': 'list',
                            'name': 'is_ok',
                            'message': f"Check the job {job_name} in jenkins. Is it ok to proceed?",
                            'choices': [
                                'yes',
                                'no',
                                "don't ask me again!"
                            ]
                        }
                    ]
                    answers = prompt(is_okay_to_proceed, style=style)
                    if answers.get("is_ok") == "no":
                        break
                    else:
                        if answers.get("is_ok") == "don't ask me again!":
                            is_prompt_user = False



import sys

import requests


def find_urls(answers):
    source_org = answers.get("source_org")
    path_to_data = answers.get("path_to_data")
    token = answers.get("github_token")

    with open("new_references.csv", "w") as writeFile:
        writeFile.write("Old URL, New URL\n")
        with open(path_to_data, "r") as listOfFiles:
            for repo in listOfFiles:
                repo = repo.replace("\n", "")
                repo_name = repo.replace(".git", "").replace(f"https://github.com/{source_org}/", "").replace("\n", "")
                print(f"Getting new url for {repo_name}")
                url = f"https://api.github.com/repos/{source_org}/{repo_name}"

                headers = {
                    'Authorization': f'token {token}',
                    'Accept': 'application/vnd.github.nebula-preview+json',

                }
                response = requests.request("GET", url, headers=headers)
                html_url = response.json().get("html_url")

                writeFile.write(f"{repo},{html_url}.git\n")

        print("Completed process. The new git Urls can be found at new_references.csv")



import json
import traceback

import time
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
import logging
import http.client
import ssl

from PyInquirer import style_from_dict, Token, prompt

logging.basicConfig(level=logging.INFO)

style = style_from_dict({
    Token.QuestionMark: '#E91E63 bold',
    Token.Selected: '#673AB7 bold',
    Token.Instruction: '',  # default
    Token.Answer: '#2196f3 bold',
    Token.Question: '',
})

accept_header = "application/vnd.github.mercy-preview+json"

errors = []

s = requests.Session()
retries = Retry(total=5, backoff_factor=1, status_forcelist=[404, 422])
s.mount('https://', HTTPAdapter(max_retries=retries))


def prompt_questions(repo, proceed_with_prompt):
    if proceed_with_prompt:
        proceed = [
            {
                'type': 'list',
                'name': 'is_ok',
                'message': f"Check the  {repo} in target repo. Is it ok to proceed To next repo?",
                'choices': [
                    'yes',
                    'no',
                    "don't ask me again!"
                ]
            }
        ]

        proceed_answers_ok = prompt(questions=proceed, style=style)

        return proceed_answers_ok
    else:
        proceed_answers_ok = {}
        proceed_answers_ok["is_ok"] = "don't ask me again!"
        return proceed_answers_ok


def get_team(answers_github):
    target_team_name = answers_github.get("target_team_name")
    source_org = answers_github.get("source_github_org")
    target_org = answers_github.get("target_github_org")
    team_url = f"https://api.github.com/orgs/{target_org}/teams/{target_team_name}"
    password = answers_github.get("github_password")
    team_resp = s.get(team_url,
                      headers={"Authorization": "token " + password,
                               "Accept": accept_header})
    team_ids = []
    if team_resp.status_code == 200:
        team_id = json.loads(team_resp.content.decode()).get("id")
        team_ids.append(team_id)
        if "parent" in json.loads(team_resp.content.decode()) and json.loads(team_resp.content.decode())[
            "parent"] is not None:
            parent_id = json.loads(team_resp.content.decode())["parent"]["id"]
            team_ids.append(parent_id)

    else:
        print("Team not found")
        exit(1)

    return team_ids


def search_by_topic(topic, source_org, password, answers_github, user_found):
    results = []

    print("Searching by topic...")

    topics = topic.split(",")

    topics = topic.split(",")
    list_of_topics = [f"topic:{single_topic}" for single_topic in topics]

    separator = "+"
    search_string = separator.join(list_of_topics)

    url = f"https://api.github.com/search/repositories?q=org:{source_org} {search_string}"

    team_ids = get_team(answers_github)

    headers = {"Authorization": "token " + password,
               "Accept": "application/vnd.github.mercy-preview+json"}

    repo_response = s.get(
        url,
        headers=headers)

    repos_contents = repo_response.content.decode("utf-8")

    if repo_response.status_code == 200:
        content = json.loads(repos_contents)
        count = json.loads(repos_contents).get("total_count")

        results.extend(content.get("items"))

        while "next" in repo_response.links:
            url = repo_response.links["next"]["url"]
            repo_response = s.get(url, headers=headers)
            data = repo_response.json()
            results.extend(data.get("items"))

        print("\n Found " + str(len(results)) + " repos with topic " + topic + " in the org " + source_org)
        questions = [
            {
                'type': 'confirm',
                'name': 'listRepos',
                'message': 'List repos found?',
                'default': False
            }, ]
        answers = prompt(questions, style=style)

        if answers.get('listRepos'):
            for item in results:
                print(item.get("full_name"))

                questions = [
                    {
                        'type': 'confirm',
                        'name': 'proceedToTransfer',
                        'message': 'If the list of repos are correct, can we proceed to transfer the repos? ',
                        'default': False
                    }, ]
            proceed_answers = prompt(questions, style=style)
        else:
            proceed_answers = {"proceedToTransfer": True}

        if proceed_answers.get("proceedToTransfer"):
            proceed_with_prompt = True
            for item in content.get("items"):
                try:
                    repo = item.get("name")
                    transfer_repo(item.get("name"), answers_github, user_found)
                    print(f"Completed transfer for " + item.get("name"))
                    proceed_answers_ok = prompt_questions(repo, proceed_with_prompt)

                    if proceed_answers_ok.get("is_ok") == "yes":
                        continue
                    elif proceed_answers_ok.get("is_ok") == "no":
                        break
                    elif proceed_answers_ok.get("is_ok") == "don't ask me again!":
                        proceed_with_prompt = False

                except Exception as ex:
                    print("Failed to transfer due to " + str(ex))
                    traceback.print_exc()

                    proceed_answers_ok = prompt_questions(repo, proceed_with_prompt)

                    if proceed_answers_ok.get("is_ok") == "yes":
                        continue
                    elif proceed_answers_ok.get("is_ok") == "no":
                        break
                    elif proceed_answers_ok.get("is_ok") == "don't ask me again!":
                        proceed_with_prompt = False
        else:
            print("Re-run the tool with correct search criteria!!")
            exit(1)
    else:
        print("No Repos found! Try running the tool again with correct topic ")


def rename_repo(repo, org, answers, headers):
    team_prefix = answers.get("prefix")
    print(f"Renaming {repo} to {team_prefix}.{repo}")
    url = f"https://api.github.com/repos/{org}/{repo}"

    data = {"name": team_prefix + "." + repo}
    rename_response = s.patch(url=url, headers=headers,
                              data=json.dumps(data))

    if rename_response.status_code == 200:
        print("Renamed successfully")
        return True

    else:
        print(f"Renaming {repo} failed due to error {str(rename_response.status_code)}")
        errors.append(repo + "| Renaming failed")
        return False


def rename_master_to_main(repo, org, answers, headers):
    print("Renaming master to main")
    team_prefix = answers.get("prefix")

    branch_url = f"https://api.github.com/repos/{org}/{team_prefix}.{repo}/branches/master/rename"
    data = {"new_name": "main"}

    rename_response = s.post(url=branch_url, headers=headers,
                             data=json.dumps(data))
    if rename_response.status_code == 201:
        print("Renamed master to main")
        return True
    else:
        print(f"Renaming to master failed for repo {repo}")
        errors.append(repo + "| Renaming master failed")
        return False


def make_repo_internal(repo, answers, headers):
    team_prefix = answers.get("prefix")
    target_org = answers.get("target_github_org")
    url = f"https://api.github.com/repos/{target_org}/{team_prefix}.{repo}"

    private_payload = json.dumps({
        "private": True,
        "visibility": "internal"
    })

    headers["Accept"] = "application/vnd.github.nebula-preview+json"

    response = requests.request("PATCH", url, headers=headers, data=private_payload)

    if response.status_code == 200:
        print(f"Made {repo} internal after transfer")


def update_webhooks(repo, org, answers, headers):
    # print("Updating Web Hooks .... ")
    #
    # team_prefix = answers.get("prefix")
    # ctx = ssl.create_default_context()
    # ctx.check_hostname = False
    # ctx.verify_mode = ssl.CERT_NONE
    # headers["User-Agent"] = "python"
    # conn = http.client.HTTPSConnection("api.github.com", context=ctx)
    #
    # url = f"/repos/{org}/{team_prefix}.{repo}/hooks"
    #
    # payload = ''
    # conn.request("GET", url, payload, headers)
    #
    # hooks_response = conn.getresponse()
    # data = hooks_response.read()
    #
    # # print(data.decode("utf-8"))
    #
    # if hooks_response.status == 200:
    #     response_json = json.loads(data.decode("utf-8"))
    #
    #     for config in response_json:
    #         hook_url = config.get("config").get("url")
    #         hook_id = config.get("id")
    #         if "bmx" in hook_url:
    #             new_hook_url = f"https://github-webhooks.baat-tools-prod.nikecloud.com/v1/{hook_url}"
    #             url = f"https://api.github.com/repos/{org}/{team_prefix}.{repo}/hooks/{hook_id}"
    #             data = {"config": {"url": new_hook_url}}
    #             response = s.patch(url, data=json.dumps(data),
    #                                headers=headers)
    #             if response.status_code == 200:
    #                 print("Updated Web Hooks ")
    #
    #         else:
    #             print(f"not a bmx repo wehbook {hook_url}")
    #     return True
    # else:
    #     print("no webhooks found")
    return True


def transfer_repo_to_targetorg(repo, answers, headers):
    source_org = answers.get("source_github_org")
    target_org = answers.get("target_github_org")
    team_prefix = answers.get("prefix")
    team_ids = get_team(answers)

    print(f"Transferring repo {repo} to {target_org}")
    url = f"https://api.github.com/repos/{source_org}/{team_prefix}.{repo}/transfer"

    data = {"new_owner": target_org, "team_ids": team_ids}
    time.sleep(0.2)
    response = s.post(url=url, headers=headers,
                      data=json.dumps(data))
    if response.status_code == 202:
        return True
    else:
        errors.append(repo + "| Failed to transfer")
        return False


def make_team_admin_writers(repo, answers, headers):
    print("changing team level access")
    team_slug = answers.get("target_team_name")
    team_admin_slug = answers.get("target_team_admin")
    team_prefix = answers.get("prefix")

    target_org = answers.get("target_github_org")

    team_url = f"https://api.github.com/orgs/{target_org}/teams/{team_slug}"

    team_resp = s.get(team_url,
                      headers=headers)
    parent_slug = team_prefix
    if team_resp.status_code == 200 and "parent" in json.loads(team_resp.content.decode()) and \
            json.loads(team_resp.content.decode())["parent"] is not None:
        parent_slug = json.loads(team_resp.content.decode())["parent"]["slug"]

    access_levels = {
        parent_slug: "push",
        team_slug: "maintain",
        team_admin_slug: "admin",
        f"{team_prefix}-external-contributors": "push"

    }

    for slug, access_level in access_levels.items():

        team_access_url = f"https://api.github.com/orgs/{target_org}/teams/{slug}/repos/{target_org}/{team_prefix}.{repo}"

        payload = json.dumps({
            "permission": access_level
        })

        response = s.put(team_access_url, headers=headers, data=payload)

        if response.status_code == 204:
            print(f"Changed {slug} to be a {access_level}")
        else:
            print(response.content.decode())
            print(f"Team access failed with error code " + str(response.status_code) + " url is " + team_access_url)
        time.sleep(0.5)


def add_user_as_admin(repo, answers, headers, user):
    print(f"Adding user {user} as adming to repo {repo}")
    source_org = answers.get("source_github_org")
    permissions_url = f"https://api.github.com/repos/{source_org}/{repo}/collaborators/{user}"

    response = s.put(url=permissions_url, headers=headers, data=json.dumps({"permission": "admin"}))

    if response.status_code == 201 or response.status_code == 204:
        print(f"User {user} added as admin to the repo {repo}")
    else:
        print(f"unable to add {user} as admin to the repo {repo} due to error {response.content.decode()}")


def remove_user_as_admin(repo, answers, headers, user):
    print(f"Removing user {user} as adming to repo {repo}")
    source_org = answers.get("source_github_org")
    permissions_url = f"https://api.github.com/repos/{source_org}/{repo}/collaborators/{user}"

    response = s.delete(url=permissions_url, headers=headers)

    if response.status_code == 201 or response.status_code == 204:
        print(f"User {user} removed as admin to the repo {repo}")
    else:
        print(f"unable to remove {user} as admin to the repo {repo} due to error {response.content.decode()}")


def add_branch_protection_main(repo, answers, headers):
    try:

        target_org = answers.get("target_github_org")
        team_slug = answers.get("target_team_name")

        url = f"https://api.github.com/repos/{target_org}/{repo}/branches/main/protection"

        payload = json.dumps({
            "required_status_checks": {
                "strict": True,
                "contexts": [
                    "continuous-integration/jenkins/pr-merge",
                    "continuous-integration/jenkins/branch"
                ]
            },
            "enforce_admins": False,
            "required_pull_request_reviews": {
                "dismissal_restrictions": {
                    "teams": [
                        team_slug
                    ]
                },
                "dismiss_stale_reviews": True,
                "require_code_owner_reviews": True,
                "required_approving_review_count": 2
            },
            "restrictions": None
        })
        headers['Accept'] = 'application/vnd.github.luke-cage-preview+json'

        response = s.put(url, headers=headers, data=payload)

        if response.status_code == 200:
            print("Updated branch protection rules")

    except Exception as ex:
        print("Updating Branch protection rules Failed ")


def transfer_repo(repo, answers, user_found):
    print("\nTransferring " + repo + " to target organization")
    source_org = answers.get("source_github_org")
    target_org = answers.get("target_github_org")

    headers = {"Authorization": "token " + answers.get('github_password'),
               "Accept": "application/vnd.github.mercy-preview+json"}

    # add as admin

    add_user_as_admin(repo, answers, headers, user_found)

    # Rename Repo
    is_rename_success = rename_repo(repo, source_org, answers, headers)

    if is_rename_success:
        is_branch_rename_success = rename_master_to_main(repo, source_org, answers, headers)
        if is_branch_rename_success:
            is_update_webhook_success = update_webhooks(repo, source_org, answers, headers)
            if is_update_webhook_success:
                is_transfer_success = transfer_repo_to_targetorg(repo, answers, headers)
                if is_transfer_success:
                    print(f"Transfer completed for repo {repo}")
                    # add status checks

                    # add branch protection rules
                    add_branch_protection_main(repo, answers, headers)
                    make_team_admin_writers(repo, answers, headers)
                    make_repo_internal(repo, answers, headers)
                    remove_user_as_admin(repo, answers, headers, user_found)
                else:
                    print(f"Transfer failed for repo {repo}")
            else:
                print(f"updating webhooks failed for repo {repo}")
        else:
            print(f"Renaming master failed for repo {repo}")
    else:
        print(f"Renaming failed for {repo} proceeding to other")


def migrate_github(answers, topic_answers):
    headers = {"Authorization": "token " + answers.get('github_password'),
               "Accept": "application/vnd.github.mercy-preview+json"}

    get_user_url = "https://api.github.com/user"

    get_user_response = requests.get(url=get_user_url, headers=headers)

    user_found = get_user_response.json().get("login")

    print(f"Logged in as user {user_found}")

    if "topic_name" in topic_answers:
        topic = topic_answers.get("topic_name")
        print("searching for repos with topic name " + topic)
        search_by_topic(
            topic, answers.get('source_github_org'), answers.get('github_password'),
            answers, user_found
        )
    else:
        filepath = topic_answers.get("filepath")
        proceed_with_prompt = True
        print("Reading repos from " + filepath)
        team_ids = get_team(answers)
        with open(filepath, 'r') as inputFile:
            for line in inputFile.readlines():
                repo_name = line.replace("\n", "")
                transfer_repo(repo_name, answers, user_found)
                print(f"Completed transfer for {repo_name}")

                if proceed_with_prompt:
                    proceed = [
                        {
                            'type': 'list',
                            'name': 'is_ok',
                            'message': f"Check the  {repo_name} in target repo. Is it ok to proceed To next repo?",
                            'choices': [
                                'yes',
                                'no',
                                "don't ask me again!"
                            ]
                        }
                    ]

                    proceed_answers_ok = prompt(questions=proceed, style=style)

                    if proceed_answers_ok.get("is_ok") == "yes":
                        continue
                    elif proceed_answers_ok.get("is_ok") == "no":
                        break
                    elif proceed_answers_ok.get("is_ok") == "don't ask me again!":
                        proceed_with_prompt = False



import requests
import json
import sys


def make_private(repo_name, answers):
    source_org = answers.get("source_org")
    token = answers.get("github_token")
    url = f"https://api.github.com/repos/{source_org}/{repo_name}"

    private_payload = json.dumps({
        "private": True,
        "visibility": "private"
    })
    headers = {
        'Authorization': f'token {token}',
        'Accept': 'application/vnd.github.nebula-preview+json',
        'Content-Type': 'application/json'
    }

    response = requests.request("PATCH", url, headers=headers, data=private_payload)

    if response.status_code == 200:
        print(f"Made {repo_name} private for transfer")


def make_private_flow(answers: dict):
    print(answers)
    source_org = answers.get("source_org")
    token = answers.get("github_token")

    url = f"https://api.github.com/orgs/{source_org}/repos?per_page=100&page=1"
    results = []
    payload = {}
    headers = {
        'Authorization': f'token {token}',
        'Accept': 'application/vnd.github.nebula-preview+json',
        'Cookie': 'logged_in=no'
    }

    response = requests.request("GET", url, headers=headers, data=payload)

    results.extend(response.json())

    while "next" in response.links:
        url = response.links["next"]["url"]

    response = requests.get(url, headers=headers)
    data = response.json()
    results.extend(data)

    with open("data_repos.json", "w") as outfile:
        outfile.write(json.dumps(results))

    for repo in results:
        if repo.get("visibility") == "internal":
            repo_name = repo.get("name")
            print(f"{repo_name} is internal, making it private")
            make_private(repo_name=repo.get("name"), answers=answers)



from setuptools import setup
import sys

from pathlib import Path  # noqa E402

CURRENT_DIR = Path(__file__).parent


def get_long_description() -> str:
    readme_md = CURRENT_DIR / "README.md"
    with open(readme_md, encoding="utf8") as ld_file:
        return ld_file.read()


version = "1.0.0"

setup(
    name="ghec_migration",
    version = "2.0.3",
    description="A Python script for bulk migrating repos from one org to other org "
                "GitHub repositories using the API",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Vamsi Maringanti",
    author_email="vamsikrishna.maringanti@nike.com",
    url="https://github.nike.com/mp-commerce-fulfillment/githubmigration",
    license="MIT",
    py_modules=["jenkins_migration", "github_migration", "ghec_migration", "make_private", "find_new_git_url", "update_topics_for_team"],
    python_requires=">=3.6",
    zip_safe=False,
    install_requires=["requests",
                      "regex",
                      "PyInquirer",
                      "python-jenkins","pbr", "multi-key-dict"],

    entry_points={
        "console_scripts": [
            "ghec-migration=ghec_migration:main"
        ]
    },
)



import requests
import sys

import json

"""[{
                'type': 'input',
                'name': 'source_org',
                'message': 'Organization Name',
                'default': "nike-platform-logistics"
            },
                {
                    'type': 'input',
                    'name': 'team_slug',
                    'message': 'Team Slug',
                    'default': "logistics-squad-airmax"
                },
                {
                    'type': 'list',
                    'name': 'team_slug',
                    'message': 'Select the permission level',
                    'choices': [
                        'push',
                        'pull',
                        'admin',
                        'maintain'
                    ]
                },
                {
                    'type': 'input',
                    'name': 'additional_topics',
                    'message': 'Additional topics, comma separated',
                    'default': "logistics,airmax"
                },
                {
                    'type': 'password',
                    'name': 'github_token',
                    'message': 'github personal access token'
                }]"""


def update_topics(answers: dict):
    source_org = answers.get("source_org")
    team_slug = answers.get("team_slug")
    permission_level = answers.get("permission_level")
    token = answers.get("github_token")
    topics_csv_string = answers.get("additional_topics")

    if permission_level not in ["push", "pull", "maintain", "admin", "triage"]:
        print("Permission level provided is invalid. "
              "It should be one of [\"push\", \"pull\", \"maintain\", \"admin\", \"triage\"]")
        exit(1)

    url = f"https://api.github.com/orgs/{source_org}/teams/{team_slug}/repos?page=1&per_page=100"
    results = []
    payload = {}
    headers = {
        'Accept': 'application/vnd.github.v3+json',
        'Authorization': f'token {token}'
    }

    repo_response = requests.request("GET", url, headers=headers, data=payload)
    repos_contents = repo_response.content.decode("utf-8")

    if repo_response.status_code == 200:
        content = json.loads(repos_contents)
        results.extend(content)

        while "next" in repo_response.links:
            url = repo_response.links["next"]["url"]
            repo_response = requests.get(url, headers=headers)
            data = repo_response.json()
            results.extend(data)

    print("Total of " + str(len(results)) + " found")
    repo_results = []
    print("Filtering the repos...")
    for repo in results:
        rep_name = repo.get("name")
        get_team_level_url = f"https://api.github.com/orgs/{source_org}/teams/{team_slug}/repos/{source_org}/{rep_name}"
        headers["Accept"] = "application/vnd.github.v3.repository+json"
        team_response = requests.request("GET", get_team_level_url, headers=headers, data=payload)
        if team_response.status_code == 200:
            if team_response.json().get("permissions").get(permission_level):
                repo_results.append(rep_name)
        else:
            print(team_response.status_code)

    print(f"Found {str(len(repo_results))} repos where {team_slug} has {permission_level} access")
    for repo in repo_results:

        topic_url = f"https://api.github.com/repos/{source_org}/{repo}/topics"
        headers["Accept"] = "application/vnd.github.mercy-preview+json"
        topics = [team_slug]
        topics.extend(topics_csv_string.split(","))
        payload = {"names": topics}
        topics_response = requests.request("PUT", topic_url, headers=headers, data=json.dumps(payload))
        if topics_response.status_code == 200:
            print(f"topics updated for {repo}")
        else:
            print(topics_response.content.decode())
